/*
  Copyright (c) 2008-2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef AUTHENTICATIONHANDLER_H__
#define AUTHENTICATIONHANDLER_H__

#include "macros.h"

#include <string>

namespace gloox
{
  class JID;
}

namespace glooxd
{

  /**
   * @brief
   *
   * @author Jakob Schroeter <js@camaya.net>
   */
  class GLOOXD_API AuthenticationHandler
  {
    public:
      /**
       * Virtual destructor.
       */
      virtual ~AuthenticationHandler() {}

      /**
       * This function is called if the server need to determine if a given user account
       * exists. The function only gets called if there is no
       * open session associated with the JID.
       * @param bare The user account in 'bare JID' form.
       * @return @b True if the account exists, @b false otherwise.
       */
      virtual bool handleUserRequest( const gloox::JID& bare ) = 0;

      /**
       * This function will be called for SASL PLAIN authentication
       * requests.
       * @param authzid The authzid.
       * @param authcid The authcid (username).
       * @param password The password.
       * @return @b True if username/password OK, @b false otherwise.
       */
      virtual bool handleSASLPlain( const std::string& authzid,
                                    const std::string& authcid,
                                    const std::string& password ) = 0;

  };

}

#endif // AUTHENTICATIONHANDLER_H__
