/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#include "certificategenerator.h"

#include "config.h"

#ifdef HAVE_OPENSSL
#include <openssl/pem.h>
#include <openssl/conf.h>
#include <openssl/x509v3.h>
#ifndef OPENSSL_NO_ENGINE
#include <openssl/engine.h>
#endif
#endif

namespace glooxd
{

  namespace util
  {

//     bool generatePrivateKey( const std::string& path )
//     {
//       (void)path;
//       return false;
//     }

    bool generateCertificate( const std::string& domain,
                              const std::string& key,
                              const std::string& path,
                              int bits, int days, int serial )
    {
      if( domain.empty() || key.empty() || path.empty()
          || bits < 1024 || days < 0 || serial <= 0 )
        return false;

#ifdef HAVE_OPENSSL
      CRYPTO_mem_ctrl( CRYPTO_MEM_CHECK_ON );

      X509* x = X509_new();
      FILE* fp = fopen( key.c_str(), "r" );
      if( !fp )
        return false;

      EVP_PKEY* pk = PEM_read_PrivateKey( fp, 0, 0, 0 );
      fclose( fp );
      fp = 0;

      if( !pk )
      {
        printf( "PEM_read_PrivateKey failed\n" );
        return false;
      }

      if( !x )
      {
        printf( "X509_new failed\n" );
        return false;
      }

      X509_set_version( x, 2 );
      ASN1_INTEGER_set( X509_get_serialNumber( x ), serial );

      X509_gmtime_adj( X509_get_notBefore( x ), 0 );
      X509_gmtime_adj( X509_get_notAfter( x ),
                       static_cast<long>( 60 * 60 * 24 * days ) );
      X509_set_pubkey( x, pk );

      X509_NAME* name = X509_get_subject_name( x );
      X509_NAME_add_entry_by_txt( name, "CN", MBSTRING_ASC,
                                  reinterpret_cast<const unsigned char*>( domain.c_str() ),
                                  -1, -1, 0 );
      X509_set_issuer_name( x, name );

      if( !X509_sign( x, pk, EVP_sha1() ) )
      {
        printf( "X509_sign failed\n" );
        return false;
      }

      fp = fopen( path.c_str(), "w" );
      if( !fp )
        return false;

      PEM_write_X509( fp, x );
      fclose( fp );
      return true;
#else
      return false;
#endif
    }

  }

}
