/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef CERTIFICATEGENERATOR_H__
#define CERTIFICATEGENERATOR_H__

#include "macros.h"

#include <string>

namespace glooxd
{

  /**
   * A namespace holding utility functions.
   */
  namespace util
  {
    /**
     * Generates a private key and stores it in the goven location.
     * @param path The path to the resulting key.
     */
//     GLOOXD_API bool generatePrivateKey( const std::string& path );

    /**
     * Generates a new self-signed certificate from the given key,
     * using the given name as the COMMON NAME, and storing it
     * in the given file.
     * @param domain The COMMON NAME for which the certificate is generated.
     * @param key The path to the private key.
     * @param path The path to the resulting certificate.
     * @param bits The 'modulo size' of the generated RSA key.
     * @param days The validity of the certificate in days.
     * @param serial A serial number.
     */
    GLOOXD_API bool generateCertificate( const std::string& domain,
                                         const std::string& key,
                                         const std::string& path,
                                         int bits, int days,
                                         int serial );

  }

}

#endif // CERTIFICATEGENERATOR_H__
