/*
  Copyright (c) 2008-2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef COMPONENTBASE_H__
#define COMPONENTBASE_H__

#include "confighandler.h"
#include "configmanager.h"
#include "macros.h"
#include "taghandler.h"
#include "router.h"

#include <gloox/jid.h>
#include <gloox/tag.h>

#include <string>

namespace gloox
{
  class LogSink;
}

namespace glooxd
{

  class Router;

  /**
   * @brief An abstract base class for the server's components.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API ComponentBase : public TagHandler, public ConfigHandler
  {
    public:
      /**
       * Constructor.
       * @param _router The local Router instance.
       * @param cm The local ConfigManager.
       * @param _name The component's internal name.
       * @param _logInstance The LogSink to use.
       */
      ComponentBase( Router& _router, ConfigManager& cm, const gloox::JID& _name,
                     const gloox::LogSink& _logInstance )
        : m_name( _name ), m_router( _router ), m_cm( cm ), m_logInstance( _logInstance )
      {
        m_router.registerDomain( this, m_name.full(), false );
        m_cm.registerConfigHandler( this );
      }

      /**
       * Virtual destructor.
       */
      virtual ~ComponentBase() {}

      /**
       * Returns the component's name by which it is addressable.
       * @return The component's name.
       */
      const gloox::JID& name() const { return m_name; }

      // reimplemented from TagHandler
      virtual void handleIncomingTag( gloox::Tag* tag ) = 0;

      // reimplemented from TagHandler
      virtual void handleOutgoingTag( gloox::Tag* tag ) = 0;

    protected:
      /** Internal name. */
      gloox::JID m_name;
      /** Local router. */
      Router& m_router;
      /** Local ConfigManager. */
      ConfigManager& m_cm;
      /** Local LogSink. */
      const gloox::LogSink& m_logInstance;

    private:
      ComponentBase( const ComponentBase& );

  };

}

#endif // COMPONENTBASE_H__
