/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef CONFIGHANDLER_H__
#define CONFIGHANDLER_H__

#include "macros.h"

#include <string>

namespace glooxd
{

  /**
   * @brief An interface that can be re-implemented to receive
   * configuration updates.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API ConfigHandler
  {
    public:
      /**
       * Virtual destructor.
       */
      virtual ~ConfigHandler() {}

      /**
       * Adds a new C2S interface.
       * @param interface The local interface to listen on.
       * @param port The local port to listen on.
       */
      virtual bool handleC2SInterfaceAdded( const std::string& interface,
                                            int port )
      {
        (void)interface;
        (void)port;
        return true;
      }

      /**
       * Removes a C2S interface.
       * @param interface The local interface to stop listening on.
       * @param port The local port to stop listening on.
       */
      virtual bool handleC2SInterfaceRemoved( const std::string& interface,
                                              int port )
      {
        (void)interface;
        (void)port;
        return true;
      }

      /**
       * This function is called if a new domain has been added that
       * the server supports.
       * @param domain The new domain.
       */
      virtual void handleDomainAdded( const std::string& domain ) = 0;

      /**
       * This function is called if a domain has been removed.
       * @param domain The removed domain.
       */
      virtual void handleDomainRemoved( const std::string& domain ) = 0;

      /**
       * This function is called to prepare a server shutdown.
       */
      virtual void handleShutdown() = 0;

  };

}

#endif // CONFIGHANDLER_H__
