/*
  Copyright (c) 2008-2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef DOMAINHANDLER_H__
#define DOMAINHANDLER_H__

#include "glooxd.h"

#include <gloox/taghandler.h>

#include <string>

namespace glooxd
{

  /**
   * @brief A base class for entities that handle Tags for a specific domain.
   *
   * @author Jakob Schroeter <js@camaya.net>
   */
  class GLOOXD_API DomainHandler : public gloox::TagHandler
  {
    public:
      /**
       * Virtual destructor.
       */
      virtual ~DomainHandler() {}

      /**
       * Re-implement this function if you want to enable other DomainHandlers to register with
       * this DomainHandler instance.
       * @param dom The domain the DomainHandler will handle.
       * @param dh The pointer to the DomainHandler for the given domain.
       */
      virtual void registerDomainHandler( const std::string& dom, DomainHandler* dh )
      {
        (void)dom;
        (void)dh;
      }

      /**
       * Returns the domain this DomainHandler can handle Tags for.
       * @param domain The domain.
       */
      virtual const std::string& domain() const { return m_domain; }

      // re-implemented from gloox::TagHandler
      virtual void handleTag( gloox::Tag* tag ) = 0;

    private:
      std::string m_domain;

  };

}

#endif // DOMAINHANDLER_H__
