/*
  Copyright (c) 2008 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef SIMPLESERVER_H__
#define SIMPLESERVER_H__

#include "../authenticationhandler.h"
#include "../c2s.h"
#include "../certificateprovider.h"
#include "../rosterprovider.h"

#include <gloox/loghandler.h>
#include <gloox/jid.h>
#include <gloox/logsink.h>

#include <string>

/**
  * @brief Implements a simple server using glooxd's components.
  *
  * @author Jakob Schroeter <js@camaya.net>
  */
class GLOOXD_API SimpleServer : gloox::LogHandler, glooxd::AuthenticationHandler, glooxd::RosterProvider,
                                glooxd::CertificateProvider
{
  public:
    /**
      * Constructor.
      */
    SimpleServer();

    /**
      * Destructor.
      */
    ~SimpleServer();

    // reimplemented from AuthenticationHandler
    virtual bool handleUserRequest( const gloox::JID& bare );

    // reimplemented from AuthenticationHandler
    virtual bool handleSASLPlain( const std::string& authzid,
                                  const std::string& authcid,
                                  const std::string& password );

    // reimplemented from gloox::LogHandler
    virtual void handleLog( gloox::LogLevel level, gloox::LogArea area, const std::string& message );

    // reimplemented from RosterProvider
    virtual const glooxd::Roster getRoster( const gloox::JID& jid );

    // reimplemented from CertificateProvider
    virtual const std::string handleCertificateRequest( const std::string& domain );

    virtual const std::string handleKeyRequest( const std::string& domain );

  private:
    gloox::LogSink m_log;

};

#endif // SIMPLESERVER_H__
