/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef FEATURECOMPRESSION_H__
#define FEATURECOMPRESSION_H__

#include "glooxd.h"
#include "streamfeature.h"

namespace glooxd
{

  /**
   * @brief An abstraction of the STARTTLS stream feature.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API FeatureCompression : public StreamFeature
  {
    public:
      /**
       * Constructor.
       */
      FeatureCompression();

      /**
       * virtual destructor.
       */
      virtual ~FeatureCompression() {}

      // reimplemented from StreamFeature
      virtual const std::string& filterString() const;

      // reimplemented from StreamFeature
      virtual gloox::Tag* tag( int state, const std::string& domain );

      // reimplemented from StreamFeature
      virtual StreamFeature* newInstance() const
      {
        return new FeatureCompression();
      }

      // reimplemented from StreamFeature
      virtual void handleTag( gloox::Tag* tag );

    private:
      enum State
      {
        ZlibOffered,
        ZlibEnabled,
        ZlibCanceled
      };

      State m_state;

  };

}

#endif // FEATURECOMPRESSION_H__
