/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/

#include "client.h"
#include "featureresourcebind.h"
#include "sm.h"

#include <gloox/adhoc.h>
#include <gloox/dataform.h>
#include <gloox/gloox.h>
#include <gloox/iq.h>

namespace glooxd
{

  FeatureResourceBind::FeatureResourceBind( SM& sm )
    : m_sm( sm ), m_state( BindCanceled )
  {
  }

  const std::string& FeatureResourceBind::filterString() const
  {
    static const std::string filter = "/iq/bind[@xmlns='" + gloox::XMLNS_STREAM_BIND + "']";
    return filter;
  }

  gloox::Tag* FeatureResourceBind::tag( int state, const std::string& /*domain*/ )
  {
    gloox::Tag* t = 0;

    if( ( state & StateAuthentication ) == StateAuthentication )
    {
      t = new gloox::Tag( "bind" );
      t->setXmlns( gloox::XMLNS_STREAM_BIND );
      m_state = BindOffered;
    }

    return t;
  }

  void FeatureResourceBind::handleTag( gloox::Tag* _tag )
  {
    if( !_tag )
      return;

    if( m_state == BindOffered && _tag->name() == "iq" )
    {
      const std::string resource = _tag->findCData( "/iq/bind[@xmlns='" + gloox::XMLNS_STREAM_BIND + "']/resource" );

      gloox::JID jid( m_parent->jid().bare() );
      jid.setResource( resource );
      if( m_sm.createSession( jid, m_parent->id() ) )
      {
        m_parent->setResource( jid.resource() );
        m_state = BindDone;
        m_parent->finishStreamFeature( StateDone );

        gloox::Tag* re = new gloox::Tag( "iq" );
        re->addAttribute( gloox::TYPE, "result" );
        re->addAttribute( "id", _tag->findAttribute( "id" ) );
        gloox::Tag* b = new gloox::Tag( re, "bind" );
        b->setXmlns( gloox::XMLNS_STREAM_BIND );
        new gloox::Tag( b, "jid", jid.full() );
        m_parent->send( re );
        return;
      }
      else
        printf( "creating a session failed\n" );
    }
    else
      printf( "state doesn't match\n" );

    gloox::IQ re( gloox::IQ::Error, _tag->findAttribute( "from" ),
                  _tag->findAttribute( "id" ));
    // FIXME add gloox::Error
    m_parent->send( re );
  }

}
