/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef FEATURERESOURCEBIND_H__
#define FEATURERESOURCEBIND_H__

#include "glooxd.h"
#include "streamfeature.h"

#include <gloox/jid.h>

namespace glooxd
{

  class SM;

  /**
   * @brief An abstraction of the Resource Bind stream feature.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API FeatureResourceBind : public StreamFeature
  {
    public:
      /**
       * Constructor.
       */
      FeatureResourceBind( SM& sm );

      /**
       * virtual destructor.
       */
      virtual ~FeatureResourceBind() {}

      // reimplemented from StreamFeature
      virtual const std::string& filterString() const;

      // reimplemented from StreamFeature
      virtual gloox::Tag* tag( int state, const std::string& domain );

      // reimplemented from StreamFeature
      virtual StreamFeature* newInstance() const
      {
        return new FeatureResourceBind( m_sm );
      }

      // reimplemented from StreamFeature
      virtual void handleTag( gloox::Tag* tag );

    private:
      enum State
      {
        BindOffered,
        BindRequested,
        BindDone,
        BindCanceled
      };

      SM& m_sm;
      State m_state;

  };

}

#endif // FEATURERESOURCEBIND_H__
