/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef FEATURESASL_H__
#define FEATURESASL_H__

#include "glooxd.h"
#include "streamfeature.h"

#include <gloox/jid.h>

namespace gloox
{
  class DataForm;
}

namespace glooxd
{

  class AuthenticationHandler;

  /**
   * @brief An abstraction of the STARTTLS stream feature.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API FeatureSASL : public StreamFeature
  {
    public:
      /**
       * Constructor.
       */
      FeatureSASL( AuthenticationHandler* authHandler );

      /**
       * virtual destructor.
       */
      virtual ~FeatureSASL() {}

      // reimplemented from StreamFeature
      virtual const std::string& filterString() const;

      // reimplemented from StreamFeature
      virtual gloox::Tag* tag( int state, const std::string& domain );

      // reimplemented from StreamFeature
      virtual StreamFeature* newInstance() const
      {
        return new FeatureSASL( m_authHandler );
      }

      // reimplemented from StreamFeature
      virtual void handleTag( gloox::Tag* tag );

    private:
      void decodeSASLPLAIN( const std::string& cdata );

      enum State
      {
        SASLOffered,
        SASLChecking,
        SASLEnabled,
        SASLCanceled
      };

      State m_state;
      AuthenticationHandler* m_authHandler;

  };

}

#endif // FEATURESASL_H__
