/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/

#include "client.h"
#include "featuresession.h"

#include <gloox/gloox.h>
#include <gloox/iq.h>

namespace glooxd
{

  FeatureSession::FeatureSession()
    : m_state( SessionCanceled )
  {
  }

  const std::string& FeatureSession::filterString() const
  {
    static const std::string filter = "/iq/session[@xmlns='" + gloox::XMLNS_STREAM_SESSION + "']";
    return filter;
  }

  gloox::Tag* FeatureSession::tag( int state, const std::string& /*domain*/ )
  {
    gloox::Tag* t = 0;

    if( ( state & StateAuthentication ) == StateAuthentication )
    {
      t = new gloox::Tag( "session" );
      t->setXmlns( gloox::XMLNS_STREAM_SESSION );
      m_state = SessionOffered;
    }

    return t;
  }

  void FeatureSession::handleTag( gloox::Tag* _tag )
  {
    if( !_tag )
      return;

    if( m_state == SessionOffered )
    {
      m_state = SessionDone;
      m_parent->finishStreamFeature( StateDone );
      gloox::IQ re( gloox::IQ::Result, _tag->findAttribute( "from" ),
                    _tag->findAttribute( "id" ));
      m_parent->send( re );
    }
    else
    {
      gloox::IQ re( gloox::IQ::Error, _tag->findAttribute( "from" ),
                    _tag->findAttribute( "id" ));
      // FIXME add gloox::Error
      m_parent->send( re );
    }
  }

}
