/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef FEATURESESSION_H__
#define FEATURESESSION_H__

#include "glooxd.h"
#include "streamfeature.h"

namespace glooxd
{

  /**
   * @brief An abstraction of the Session Create stream feature.
   *
   * Basically a NOOP.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API FeatureSession : public StreamFeature
  {
    public:
      /**
       * Constructor.
       */
      FeatureSession();

      /**
       * virtual destructor.
       */
      virtual ~FeatureSession() {}

      // reimplemented from StreamFeature
      virtual const std::string& filterString() const;

      // reimplemented from StreamFeature
      virtual gloox::Tag* tag( int state, const std::string& domain );

      // reimplemented from StreamFeature
      virtual StreamFeature* newInstance() const
      {
        return new FeatureSession();
      }

      // reimplemented from StreamFeature
      virtual void handleTag( gloox::Tag* tag );

    private:
      enum State
      {
        SessionOffered,
        SessionDone,
        SessionCanceled
      };

      State m_state;

  };

}

#endif // FEATURESESSION_H__
