/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef FEATURESTARTTLS_H__
#define FEATURESTARTTLS_H__

#include "glooxd.h"
#include "streamfeature.h"

#include <gloox/tlshandler.h>

namespace gloox
{
  class Tag;
  class ConnectionTLSServer;
}

namespace glooxd
{

  class CertificateProvider;

  /**
   * @brief An abstraction of the STARTTLS stream feature.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API FeatureSTARTTLS : public StreamFeature, public gloox::TLSHandler
  {
    public:
      /**
       * Constructor.
       * @param @b True if a &lt;required/&gt; should be included
       * in the stream feature.
       * @param cp The CertificateProvider.
       */
      FeatureSTARTTLS( bool required,
                       CertificateProvider* cp );

      /**
       * virtual destructor.
       */
      virtual ~FeatureSTARTTLS();

      // reimplemented from StreamFeature
      virtual const std::string& filterString() const;

      // reimplemented from StreamFeature
      virtual gloox::Tag* tag( int state, const std::string& domain );

      // reimplemented from StreamFeature
      virtual StreamFeature* newInstance() const
      {
        return new FeatureSTARTTLS( m_required, m_cp );
      }

      // reimplemented from StreamFeature
      virtual void handleTag( gloox::Tag* tag );

      // reimplemented from gloox::TLSHandler
      virtual void handleEncryptedData( const gloox::TLSBase* /*base*/, const std::string& /*data*/ ) {}

      // reimplemented from gloox::TLSHandler
      virtual void handleDecryptedData( const gloox::TLSBase* /*base*/, const std::string& /*data*/ ) {}

      // reimplemented from gloox::TLSHandler
      virtual void handleHandshakeResult( const gloox::TLSBase* base, bool success, gloox::CertInfo &certinfo );

    private:
      enum State
      {
        TLSOffered,
        TLSHandshake,
        TLSEnabled,
        TLSCanceled
      };

      std::string m_key;
      std::string m_cert;
      State m_state;
      CertificateProvider* m_cp;
      gloox::ConnectionTLSServer* m_encryption;
      bool m_required;

  };

}

#endif // FEATURESTARTTLS_H__
