/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef PLUGIN_H__
#define PLUGIN_H__

#include "macros.h"

#include <string>

namespace gloox
{
  class Tag;
}

namespace glooxd
{

  class SM;
  class TagHandler;

  /**
   * @brief Abstract base class for a (sub)protocol such as Roster Management,
   * Private XML Storage, etc.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API Plugin
  {
    public:
      /**
       * Constructor.
       * @param sm The SM to use.
       * @param router The Router to use.
       */
      Plugin( SM& sm, TagHandler& router )
        : m_sm( sm ), m_router( router )
      {}

      /**
       * Virtual destructor.
       */
      virtual ~Plugin() {}

      /**
       * Reimplement this function to handle incoming tags matched by the
       * XPath expression returned by filterString().
       * @param tag A Tag to handle.
       * @return @b True if the Tag has been handled, @b false if it is
       * to be passed on to the next Plugin in the chain.
       */
      virtual bool handleTag( const gloox::Tag* tag ) = 0;

      /**
       * Returns an XPath expression that matches the Plugin's supported
       * stanzas.
       * @return An XPath expression.
       */
      virtual const std::string& filterString() const = 0;

    protected:
      SM& m_sm;
      TagHandler& m_router;

    private:
      Plugin( const Plugin& );

  };

}

#endif // PLUGIN_H__
