/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#include "rosterprovider.h"
#include "presencemanager.h"
#include "taghandler.h"
#include "sm.h"

#include <gloox/tag.h>

namespace glooxd
{

  PresenceManager::PresenceManager( SM& sm, TagHandler& router,
                                RosterProvider* rp )
    : Plugin( sm, router ), m_handler( rp )
  {
  }

  PresenceManager::~PresenceManager()
  {
  }

  const std::string& PresenceManager::filterString() const
  {
    static const std::string filter = "/presence";
    return filter;
  }

  void PresenceManager::getRoster( const gloox::JID& jid )
  {
    if( m_handler )
    {
      const Roster& rm = m_handler->getRoster( jid );
      Roster::const_iterator it = rm.begin();
      for( ; it != rm.end(); ++it )
      {
        switch( (*it).second )
        {
          case gloox::S10nBoth:
            break;
          case gloox::S10nNone:
            break;
          case gloox::S10nFrom:
            break;
          case gloox::S10nTo:
            break;
          default:
            break;
        }
      }
    }
  }

  bool PresenceManager::handleTag( const gloox::Tag* tag )
  {
    if( !tag || !m_handler )
      return false;

    const std::string& type = tag->findAttribute( gloox::TYPE );
    if( type == "subscribe" || type == "unsubscribe"
        || type == "subscribed" || type == "unsubscribed" )
      return false;

    printf( "received a presence tag!!!!!!!!!! %s\n", tag->xml().c_str() );

    m_sm.dispatchPresence( tag );

    return true;
  }


}
