/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef PRESENCEMANAGER_H__
#define PRESENCEMANAGER_H__

#include "glooxd.h"
#include "plugin.h"

#include <gloox/tag.h>

namespace gloox
{
  class JID;
  class Tag;
}

namespace glooxd
{

  class RosterProvider;

  /**
   * @brief An implementation of the server-side presence management.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 0.2
   */
  class GLOOXD_API PresenceManager : public Plugin
  {
    public:
      /**
       * Constructor.
       * @param sm The SM to use.
       * @param router The Router to use.
       * @param rp The roster source.
       */
      PresenceManager( SM& sm, TagHandler& router, RosterProvider* rp );

      /**
       * Virtual destructor.
       */
      virtual ~PresenceManager();

      // reimplemented from Plugin
      virtual bool handleTag( const gloox::Tag* tag );

      // reimplemented from Plugin
      virtual const std::string& filterString() const;

    private:
      void getRoster( const gloox::JID& jid );

      RosterProvider* m_handler;

  };

}

#endif // PRESENCEMANAGER_H__
