/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the glooxd library. http://camaya.net/glooxd

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#include "rosterprovider.h"
#include "rostermanager.h"
#include "taghandler.h"

#include <gloox/tag.h>

namespace glooxd
{

  RosterManager::RosterManager( SM& sm, TagHandler& router,
                                RosterProvider* rp )
    : Plugin( sm, router ), m_handler( rp )
  {
  }

  RosterManager::~RosterManager()
  {
  }

  const std::string& RosterManager::filterString() const
  {
    static const std::string filter = "/iq[/iq/query[@xmlns='" + gloox::XMLNS_ROSTER + "']]"
        "|/presence[@type='subscribe']"
        "|/presence[@type='unsubscribe']"
        "|/presence[@type='subscribed']"
        "|/presence[@type='unsubscribed']";
    return filter;
  }

  const gloox::TagList RosterManager::getRoster( const gloox::JID& jid )
  {
    gloox::TagList tl;
    if( m_handler )
    {
      const Roster& rm = m_handler->getRoster( jid );
      Roster::const_iterator it = rm.begin();
      for( ; it != rm.end(); ++it )
      {
        gloox::Tag* i = new gloox::Tag( "item" );
        i->addAttribute( "jid", (*it).first );
        switch( (*it).second )
        {
          case gloox::S10nBoth:
            i->addAttribute( "subscription", "both" );
            break;
          case gloox::S10nNone:
            i->addAttribute( "subscription", "none" );
            break;
          case gloox::S10nFrom:
            i->addAttribute( "subscription", "from" );
            break;
          case gloox::S10nTo:
            i->addAttribute( "subscription", "to" );
            break;
          default:
            break;
        }
        tl.push_back( i );
      }
    }
    return tl;
  }

  bool RosterManager::handleTag( const gloox::Tag* tag )
  {
    if( !tag )
      return false;

    printf( "received a roszter tag!!!!!!!!!! %s\n", tag->xml().c_str() );

    if( tag->hasAttribute( gloox::TYPE, "get" ) )
    {
      gloox::Tag* re = new gloox::Tag( "iq" );
      re->addAttribute( "id", tag->findAttribute( "id" ) );
      re->addAttribute( "to", tag->findAttribute( "from" ) );
      re->addAttribute( gloox::TYPE, "result" );
      gloox::Tag* q = new gloox::Tag( re, "query" );
      q->setXmlns( gloox::XMLNS_ROSTER );
      const gloox::TagList& r = getRoster( tag->findAttribute( "from" ) );
      gloox::TagList::const_iterator it = r.begin();
      for( ; it != r.end(); ++it )
        q->addChild( (*it) );

      m_router.handleIncomingTag( re );
      return true;
    }

    return false;
  }

}
