/*
  Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
  This file is part of the gloox library. http://camaya.net/gloox

  This software is distributed under a license. The full license
  agreement can be found in the file LICENSE in this distribution.
  This software may not be copied, modified, sold or distributed
  other than expressed in the named license agreement.

  This software is distributed without any warranty.
*/


#ifndef SEMAFORE_H__
#define SEMAFORE_H__

#include "macros.h"

namespace glooxd
{

  namespace util
  {
    /**
     * @brief A simple implementation of semaphore as a wrapper around a pthread or win32 semaphore.
     *
     * @author Jakob Schroeter <js@camaya.net>
     * @since 1.1
     */
    class GLOOXD_API Semaphore
    {
      public:
        /**
         * Contructs a new simple semaphore.
         * @param value The initial semaphore value.
         */
        Semaphore( int value = 0 );

        /**
         * Destructor.
         */
        ~Semaphore();

        /**
         * Waits for the semaphore.
         */
        void wait();

        /**
         * Tries to wait for the semaphore.
         * @return @b True if the semaphore could be locked,
         * @b false otherwise.
         */
        bool trywait();

        /**
         * Posts (adds 1) to the semaphore.
         */
        void post();

      private:
        class SemaphoreImpl;

        Semaphore& operator=( const Semaphore& );
        SemaphoreImpl* m_semaphore;

    };

  }

}

#endif // SEMAFORE_H__
